<?php

declare(strict_types=1);

namespace Drupal\style_options_media\Plugin\StyleOption;

use Drupal\Core\Ajax\AjaxHelperTrait;
use Drupal\Core\Form\FormStateInterface;
use Drupal\style_options\StyleOptionStyleTrait;
use Drupal\style_options\Plugin\StyleOptionPluginBase;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Define the image attribute option plugin.
 *
 * @StyleOption(
 *   id = "background_media",
 *   label = @Translation("Background media")
 * )
 */
class BackgroundMedia extends StyleOptionPluginBase {

  use AjaxHelperTrait;
  use StyleOptionStyleTrait;

  /**
   * {@inheritDoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state): array {
    $backgroundMedia = $this->getValue('background_media') ?? [];
    $form['background_media'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Background Media'),
      '#tree' => TRUE,
      'media' => [
        '#title' => t('Background Media'),
        '#default_value' => $backgroundMedia['media'] ?? NULL,
        '#description' => t('Select the media item to use as a background.'),
        '#allowed_bundles' => ['image', 'video', 'remote_video'],
        '#type' => 'media_library',
      ],
      'blend_mode' => [
        '#default_value' => $backgroundMedia['blend_mode'] ?? NULL,
        '#description' => t('Select the blend mode for the background media.'),
        '#empty_value' => '',
        '#options' => [
          'color' => t('Color'),
          'color-burn' => t('Color Burn'),
          'color-dodge' => t('Color Dodge'),
          'darken' => t('Darken'),
          'difference' => t('Difference'),
          'exclusion' => t('Exclusion'),
          'hard-light' => t('Hard Light'),
          'hue' => t('Hue'),
          'lighten' => t('Lighten'),
          'luminosity' => t('Luminosity'),
          'multiply' => t('Multiply'),
          'normal' => t('Normal'),
          'overlay' => t('Overlay'),
          'saturation' => t('Saturation'),
          'screen' => t('Screen'),
          'soft-light' => t('Soft Light'),
        ],
        '#title' => t('Blend Mode'),
        '#type' => 'select',
      ],
    ];
    return $form;
  }

  /**
   * {@inheritDoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state): void {
    $values = $form_state->cleanValues()->getValues();
    $this->setValues($values);
  }

  /**
   * {@inheritDoc}
   */
  public function build(array $build, $value = '') {
    // Adds background media if available.
    if (empty($mediaOptions = $this->getValue('background_media') ?? NULL)) {
      return $build;
    }

    $styleOptions = [];
    $build['#style_options_media'] = &$styleOptions;
    if (!empty($mediaOptions['media'] ?? NULL)) {
      $vb = $this->entityTypeManager->getViewBuilder('media');
      $media = $this->entityTypeManager->getStorage('media')->load($mediaOptions['media']);
      $styleOptions['media'] = $vb->view($media, 'background');
    }

    // Adds blend mode if available.
    if (!empty($blendMode = $mediaOptions['blend_mode'] ?? NULL)) {
      $styleOptions['blend_mode'] = $blendMode;
    }

    return $build;
  }

}
